/** ----------------------------------------------------
 ** TFC Sistemes Encastats T-2010 - Xavier Moldes 
 ** ----------------------------------------------------
 **
 ** tfcNet.h
 **
 ** Header principal del projecte. 
 ** Conté les constants i definicions globals
 ** Quan es parla d'estació es refereix a les motes que 
 ** envien dades. El Host es el bridge entre la xarxa i
 ** el PC.
 ** ---------------------------------------------------- **/
  
#ifndef TFC_NET_H
#define TFC_NET_H

/* enum amb constants integer globals */ 
enum 
{
   STATION_RADIO_TIME_WARM_UP = 100,			// Temps, en ms, que esperem abans d'arrencar la ràdio a l'estació
   STATION_RADIO_MAIN_CYCLE = 100,				// Temps del cicle d'execució de la màquina d'estats de la ràdio a l'estació
   
   HOST_RADIO_TIME_WARM_UP = 100,				// Temps, en ms, que esperem abans d'arrencar la ràdio
   HOST_RADIO_MAIN_CYCLE = 100,				  	// Temps del cicle d'execució de la màquina d'estats de la ràdio
   
   HOST_SERIAL_TIME_WARM_UP = 100,				// Temps, en ms, que esperem abans d'arrencar l'USB
   HOST_SERIAL_MAIN_CYCLE = 100,				// Temps del cicle d'execució de la màquina d'estats de l'USB
   
   SENSORS_CACHE_MAIN_CYCLE = 500,				// Temps del cicle d'execucuó de la màquina d'estats del cache
   
   TEMPERATURE_SENDER_INITIAL_DELAY = 600,		// Temps, en ms, que esperem abans d'arrencar l'enviament de temperatura
   TEMPERATURE_SENDER_MAIN_CYCLE = 5000,		// Temps del cicle d'execucuó de la màquina d'estats de l'enviament de temperatura
   
   PHOTO_SENDER_INITIAL_DELAY = 1200,			// Temps, en ms, que esperem abans d'arrencar l'enviament de lluminositat
   PHOTO_SENDER_MAIN_CYCLE = 5000,				// Temps del cicle d'execucuó de la màquina d'estats de l'enviament de lluminositat
   
   BATTERY_SENDER_INITIAL_DELAY = 1800,			// Temps, en ms, que esperem abans d'arrencar l'enviament de bateria
   BATTERY_SENDER_MAIN_CYCLE = 5000,			// Temps del cicle d'execucuó de la màquina d'estats de l'enviament de bateria
   
   HALL_SENDER_INITIAL_DELAY = 10000,			// Temps d'inici del sensor d'efecte Hall	
   
   CYCLES_WAIT_HOST = 4,						// Nombre de cicles del bucle principal de ràdio que esperem
};   											// per reintentar trobar el host
   
/* Tipus de missatge que circulen per la xarxa */
enum 
{
	HANDSHAKE_HELLO = 1001,					// Missatge de salutació
	HANDSHAKE_IAMHOST = 1002,				// Resposta a la salutació
};
enum
{		
	/* 2001 */ MSG_TEMPERATURE = 2001,				// Enviament de temperatura
	/* 2002 */ MSG_LIGHT,							// Enviament de lluminositat
	/* 2003 */ MSG_BATTERY,							// Enviament de nivell de càrrega
	/* 2004 */ MSG_HALL,							// Enviament de detecció Hall
	/* 2005 */ MSG_DO_ACTION_1,						// Simulació acció 1 (encen i apaga Led 1)
	/* 2006 */ MSG_DONE_ACTION_1,
	/* 2007 */ MSG_DO_ACTION_2,						// Simulació acció 2 (encen i apaga Led 2) 
	/* 2008 */ MSG_DONE_ACTION_2,
};

/* Canals de comunicació */
enum
{
	AM_XMLENVELOPEUSBMSG = 0x21,			// Canal d'enviament mota <-> PC
	AM_XMLENVELOPEAIRMSG = 0x22,			// Canal d'enviament mota <-> mota
};

// Estructures per l'enviament de missatge 
// Una (xmlEnvelopMsg) és la que circula per 
// la xarxa i l'altre (tfcMessage) és la que
// es fa servir dins el codi

enum 
{
	// Logitud màxima de l'string XML en bytes
	XML_BUFFER_LENGTH = 80,
};


typedef nx_struct xmlEnvelopeMsg
{
	nx_uint8_t bufferXML[XML_BUFFER_LENGTH];
	
} xmlEnvelopeMsg;

typedef nx_struct tfcMessage
{
	nx_uint16_t senderId;
	nx_uint16_t receiverId;
	nx_uint16_t infoType;
	nx_uint16_t infoData;	

} tfcMessage;

#endif /* TFC_NET_H */
