using System;
using System.IO;
using System.Net.Sockets;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.Xml;
using System.ComponentModel;
using System.Data;
using System.Net;


namespace tfcSocket
{
    public enum enTFCmessageType
    {
        /* 2001 */ MSG_TEMPERATURE = 2001,				// Enviament de temperatura
        /* 2002 */ MSG_LIGHT,							// Enviament de lluminositat
        /* 2003 */ MSG_BATTERY,							// Enviament de nivell de crrega
        /* 2004 */ MSG_HALL,							// Enviament de detecci Hall
        /* 2005 */ MSG_DO_ACTION_1,						// Simulaci acci 1 (encen i apaga Led 1)
        /* 2006 */ MSG_DONE_ACTION_1,
        /* 2007 */ MSG_DO_ACTION_2,						// Simulaci acci 2 (encen i apaga Led 2) 
        /* 2008 */ MSG_DONE_ACTION_2,
    };

    //----------------------------------------------------------------------
    #region cLectura

    [Serializable]
    public class cLectura
    {
        protected int pId;
        protected enTFCmessageType pTipus;
        protected string pDades;
        protected System.DateTime pMoment;

        public cLectura(TFCMSG msg)
        {
            pId = msg.From;
            pTipus = (enTFCmessageType)msg.Type;
            pDades = msg.Data.ToString();
            pMoment = System.DateTime.Now;
        }

        public int Id
        {
            get { return pId; }
            set { pId = value; }
        }

        public enTFCmessageType Tipus
        {
            get { return pTipus; }
            set { pTipus = value; }
        }

        public string Dades
        {
            get { return pDades; }
            set { pDades = value; }
        }

        public DateTime Moment
        {
            get { return pMoment; }
            set { pMoment = value; }
        }
    }
    #endregion
    //----------------------------------------------------------------------
    #region cStation

    [Serializable]
    public class cStation
    {
        protected int pId;
        protected TFCMSG pLastMessage;
        protected string pLastXML;
        protected cLectura [] pLectures = new cLectura[100];
        protected int pIX;
        protected int pCount;

        public cStation()
        {
            pIX = 0;
            pCount = 0;
        }

        public int Id
        {
            get { return pId; }
            set { pId = value; }
        }

        public TFCMSG LastMessage
        {
            get { return pLastMessage; }
            set 
            {
                pCount++;
                pLastMessage = value;
                cLectura lc = new cLectura(value);
                pLectures[pIX] = lc;

                if (++pIX >= 100)
                    pIX = 0;
            }
        }

        public string LastXML
        {
            get { return pLastXML; }
            set { pLastXML = value; }
        }

        public cLectura getLectura(enTFCmessageType tipus)
        {
            System.DateTime hr = System.DateTime.MinValue;
            cLectura ret = null;

            for (int i = 0; i < 100; i++)
            {
                if (pLectures[i] != null && pLectures[i].Tipus == tipus && pLectures[i].Moment > hr)
                {
                    hr = pLectures[i].Moment;
                    ret = pLectures[i];
                }
            }
            return ret;
        }
    }
    #endregion
    //----------------------------------------------------------------------
    #region cTFCmanager

    [Serializable]
    public class cTFCmanager : System.MarshalByRefObject
    {
        protected cStation[] pStations = new cStation[100];
        protected int pIX = 0;

        Thread thReceive;
        TFCMSG pending = null;

        //-----------------------------------------------------------------------------------------------------

        protected class XmlTextWriterFormattedNoDeclaration : System.Xml.XmlTextWriter
        {
            public XmlTextWriterFormattedNoDeclaration(System.IO.TextWriter w)
                : base(w)
            {
                Formatting = System.Xml.Formatting.None; //no s'inserten block returns
                //Formatting = System.Xml.Formatting.Indented; //SI s'inserten block returns
            }
            public override void WriteStartDocument() { } //no hi ha capalera
        }
        //-----------------------------------------------------------------------------------------------------

        static public string ObjToXml(TFCMSG obj)
        {
            StringBuilder sb = new StringBuilder();

            XmlSerializerNamespaces namespaces = new XmlSerializerNamespaces();
            namespaces.Add(string.Empty, string.Empty);
            XmlSerializer serializer = new XmlSerializer(typeof(TFCMSG));
            XmlTextWriterFormattedNoDeclaration writer = new XmlTextWriterFormattedNoDeclaration(new StringWriter(sb));
            serializer.Serialize(writer, obj, namespaces);
            writer.Close();

            return sb.ToString();
        }
        //-----------------------------------------------------------------------------------------------------

        static public TFCMSG XmlToObj(byte[] xml)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(TFCMSG));
            MemoryStream myxml = new MemoryStream();
            myxml.Write(xml, 0, xml.Length);
            myxml.Position = 0;
            TFCMSG obj = (TFCMSG)serializer.Deserialize(myxml);
            myxml.Close();
            return obj;
        }
        //-----------------------------------------------------------------------------------------------------

        public cTFCmanager()
        {
            pIX = 0;
        }
        //-----------------------------------------------------------------------------------------------------

        protected cStation getStation(int id, bool create)
        {
            for (int i = 0; i < pIX; i++)
                if (pStations[i].Id == id)
                    return pStations[i];

            if (create == true)
            {
                pStations[pIX] = new cStation();
                pStations[pIX].Id = id;
                return pStations[pIX++];
            }
            return null;
        }
        //-----------------------------------------------------------------------------------------------------

        public cStation getStation(int id)
        {
            return getStation(id, false);
        }
        //-----------------------------------------------------------------------------------------------------

        public void processMessage(TFCMSG msg, string check)
        {
            cStation st = getStation(msg.From, true);
            if (st != null)
            {
                st.LastMessage = msg;
                st.LastXML = check;
            }
        }
        //-----------------------------------------------------------------------------------------------------

        public void connectSF(string ip, int port)
        {
            IPEndPoint ep = new IPEndPoint(IPAddress.Parse(ip), port);
            thReceive = new Thread(new ParameterizedThreadStart(readSocket));
            thReceive.Start(ep);
        }
        //-----------------------------------------------------------------------------------------------------

        public void disconnectSF()
        {
            if (thReceive != null)
            {
                thReceive.Abort();
                thReceive.Join();
            }
        }
        //-----------------------------------------------------------------------------------------------------

        public void doAction1()
        {
            TFCMSG msg = new TFCMSG();
            msg.From = 999;
            msg.To = 1;
            msg.Type = (int)enTFCmessageType.MSG_DO_ACTION_1;
            msg.Data = "0";
            pending = msg;
        }
        //-----------------------------------------------------------------------------------------------------

        public void doAction2()
        {
            TFCMSG msg = new TFCMSG();
            msg.From = 999;
            msg.To = 1;
            msg.Type = (int)enTFCmessageType.MSG_DO_ACTION_2;
            msg.Data = "0";
            pending = msg;
        }
        //-----------------------------------------------------------------------------------------------------

        /*
        char lenmsg = 9;
        unsigned char buff[9];
        int i = 0;

        buff[i++] = 0x00;  // syncByte
        buff[i++] = 0xFF;  // Destination Address
        buff[i++] = 0xFF;  // Destination Address
        buff[i++] = 0x00;  // Link source address
        buff[i++] = 0x00;  // Link source address
        buff[i++] = 0x01;  // Message length
        buff[i++] = 0x00;  // Group ID
        buff[i++] = 0x09;  // Active Message handler type
        buff[i++] = 0x56;  // Payload

        send(fd, &lenmsg, 1, 0);    
        send(fd, (char*)buff, lenmsg, 0);
        return 0;
        */
        protected void sendPending(Socket socket)
        {
            string xml = ObjToXml(pending);
            byte[] buff = new byte[88];
            byte[] h = new byte[1];

            h[0] = 88;

            buff[0] = 0;
            buff[1] = 255;
            buff[2] = 255;
            buff[3] = 0;
            buff[4] = 0;
            buff[5] = 80;
            buff[6] = 0;
            buff[7] = 33;

            byte[] msg = System.Text.Encoding.ASCII.GetBytes(xml);

            int i;
            for (i = 0; i < msg.GetUpperBound(0); i++)
                buff[i + 8] = msg[i];

            for (; i < 80; i++)
                buff[i + 8] = 0;

            int s = socket.Send(h);
            s = socket.Send(buff);
            pending = null;
        }
        //-----------------------------------------------------------------------------------------------------

        protected void readSocket(object ip)
        {
            if (!(ip is IPEndPoint))
                return;

            Byte[] handshake = new Byte[2];
            Byte[] bytes = new Byte[255];
            Byte[] head = new Byte[1];
            int nb;
            Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            socket.Connect((IPEndPoint)ip);

            try
            {
                // Handshake
                nb = 0;
                while (nb == 0)
                {
                    Thread.Sleep(100);
                    nb = socket.Receive(handshake, handshake.Length, SocketFlags.None);
                }
                socket.Send(handshake);
                Thread.Sleep(100);

                while (true)
                {
                    do
                    {
                        while (socket.Available == 0)
                        {
                            Thread.Sleep(50);

                            if (pending != null)
                                sendPending(socket);
                        }

                        nb = socket.Receive(head, head.Length, SocketFlags.None);
                        if (head[0] > 0)
                        {
                            nb = socket.Receive(bytes, head[0], SocketFlags.None);
                            byte[] xml = new byte[nb - 8];

                            for (int j = 0; j < nb - 8; j++)
                                xml[j] = bytes[j + 8];

                            string check = Encoding.ASCII.GetString(xml, 0, nb - 8);
                            TFCMSG msg = XmlToObj(xml);
                            msg.Save();

                            processMessage(msg, check);
                            msg = null;
                            Program.pFormSocket.logText(check);
                        }

                    } while (nb > 0);
                }
            }
            catch (Exception ex)
            {
                if (ex is System.Threading.ThreadAbortException)
                {
                    socket.Close();
                }
            }
        }
    }
    #endregion
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
}
